#include "ad_da.h"
#include "jbuf.h"
#include <stdio.h>
#include <string.h>
#include "freertos/FreeRTOS.h"
#include "freertos/task.h"
#include "esp_log.h"
#include "driver/i2s.h"


static const char* TAG = "ad/da";

//i2s port number
#define EXAMPLE_I2S_NUM           (0)

#define EXAMPLE_I2S_SAMPLE_RATE   (16000)
//i2s data bits
#define EXAMPLE_I2S_SAMPLE_BITS   (16)
//I2S data format
#define EXAMPLE_I2S_FORMAT        (I2S_CHANNEL_FMT_RIGHT_LEFT)
//I2S channel number
#define EXAMPLE_I2S_CHANNEL_NUM   ((EXAMPLE_I2S_FORMAT < I2S_CHANNEL_FMT_ONLY_RIGHT) ? (2) : (1))

static int16_t stereo_buf[JBUF_FRAME_SIZE * 2];



/** @brief I2S ADC/DAC mode init.
 */
void ad_da_init(void)
{
    ESP_LOGI(TAG, "Initializing...");
    int i2s_num = EXAMPLE_I2S_NUM;
    i2s_config_t i2s_config =
    {
        .mode = I2S_MODE_MASTER | I2S_MODE_TX,
        .sample_rate =  EXAMPLE_I2S_SAMPLE_RATE,
        .bits_per_sample = EXAMPLE_I2S_SAMPLE_BITS,
        .communication_format = I2S_COMM_FORMAT_I2S | I2S_COMM_FORMAT_I2S_MSB,
        .channel_format = EXAMPLE_I2S_FORMAT,
        .intr_alloc_flags = 0,
        .dma_buf_count = 4, //2,
        .dma_buf_len = JBUF_FRAME_SIZE * sizeof(short),
        .use_apll = 1,
    };
    //install and start i2s driver
    i2s_driver_install(i2s_num, &i2s_config, 0, NULL);

    // I2S pins can be freely remapped; here they are laid identically to PCM5102 module pinout
    i2s_pin_config_t pin_config = {
            .bck_io_num = GPIO_NUM_4,
            .ws_io_num = GPIO_NUM_15,
            .data_out_num = GPIO_NUM_2,
            .data_in_num = I2S_PIN_NO_CHANGE
    };
    i2s_set_pin(EXAMPLE_I2S_NUM, &pin_config);
}

void convert_mono_to_stereo(int16_t* buf, int16_t* stereo_buf, unsigned int count)
{
    unsigned int out_pos = 0;
    for (unsigned int i=0; i<count; i++)
    {
        stereo_buf[out_pos] = buf[i];
    #if 1
        stereo_buf[out_pos+1] = buf[i+1];
    #else
        // right channel only
        stereo_buf[out_pos+1] = 0;
    #endif
        out_pos += 2;
    }
}

void i2s_adc_dac_fn(void *arg)
{
    i2s_set_clk(EXAMPLE_I2S_NUM, EXAMPLE_I2S_SAMPLE_RATE, EXAMPLE_I2S_SAMPLE_BITS, EXAMPLE_I2S_CHANNEL_NUM);
    while (1)
    {
        size_t bytes_written = 0;
        short* buf = jbuf_get();
        if (buf)
        {
            convert_mono_to_stereo(buf, stereo_buf, JBUF_FRAME_SIZE);
        }
        else
        {
            memset(stereo_buf, 0, sizeof(stereo_buf));
        }
        i2s_write(EXAMPLE_I2S_NUM, stereo_buf, sizeof(stereo_buf), &bytes_written, portMAX_DELAY);
        if (bytes_written != sizeof(stereo_buf)) {
            ESP_LOGI(TAG, "written %d", bytes_written);
        }
    }
    vTaskDelete(NULL);
}





